'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.createOnSubmitHandler = createOnSubmitHandler;
exports.createOnRemoveHandler = createOnRemoveHandler;
exports.createMapStateToPropsDetail = createMapStateToPropsDetail;
exports.connectSingleItem = connectSingleItem;

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _utils = require('../../utils');

var utils = _interopRequireWildcard(_utils);

var _common = require('./common');

/**
 * Returns onSubmit handler that dispatch `put` action
 *
 * @param {crud} crud
 * @returns {function}
 */

function createOnSubmitHandler(crud) {
  return function onSubmit(item, data, dispatch) {
    var doc = _extends({}, item, data);
    if (!doc._id) {
      doc._id = crud.mountPoint + '-' + _uuid2['default'].v4();
    }
    dispatch(crud.actions.put(doc));
  };
}

/**
 * createOnRemoveHandler
 *
 * @param crud
 * @returns {function}
 */

function createOnRemoveHandler(crud) {
  return function onRemove(dispatch, items) {
    dispatch(crud.actions.remove(items.toObject()));
  };
}

function createMapStateToPropsDetail(crud, opts, mapStateToProps) {
  if (opts === undefined) opts = {};

  return function (state, ownProps) {
    var props = mapStateToProps ? mapStateToProps(state, ownProps) : {};
    var finalOpts = Object.assign({}, opts, props.singleItemOpts, ownProps);
    var _finalOpts$propName = finalOpts.propName;
    var propName = _finalOpts$propName === undefined ? 'item' : _finalOpts$propName;
    var docId = finalOpts.docId;

    props.propName = propName;
    props[propName] = utils.getObjectFromState(state, crud.mountPoint, docId);
    props.action = crud.actions.get;
    props.actionArgs = [docId];
    return props;
  };
}

/**
 * Decorator connects passed component to include single document as `item`.
 *
 * @param {crud} crud
 */

function connectSingleItem(crud, opts, mapStateToProps, mapDispatchToProps) {
  if (opts === undefined) opts = {};

  (0, _invariant2['default'])(typeof crud !== 'undefined', 'crud is required parameter');
  var mapStateToPropsFinal = createMapStateToPropsDetail(crud, opts, mapStateToProps);
  return (0, _common.wrap)(mapStateToPropsFinal, mapDispatchToProps);
}

;