'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.toList = toList;
exports.findDocById = findDocById;
exports.findDoc = findDoc;
exports.deleteDocFromList = deleteDocFromList;
exports.getFoldersFromState = getFoldersFromState;
exports.getFolderFromState = getFolderFromState;
exports.getDocumentsFromState = getDocumentsFromState;
exports.saveFolderVars = saveFolderVars;
exports.getFolderVars = getFolderVars;
exports.saveIdsInFolder = saveIdsInFolder;
exports.getIdsFromFolder = getIdsFromFolder;
exports.hasFolder = hasFolder;
exports.invalidateFolders = invalidateFolders;
exports.getDocumentsInFolder = getDocumentsInFolder;
exports.removeDocument = removeDocument;
exports.getDocument = getDocument;
exports.setDocument = setDocument;
exports.setDocuments = setDocuments;
exports.setQueryPayloadInState = setQueryPayloadInState;
exports.setDocInList = setDocInList;
exports.createActionType = createActionType;
exports.markDocumentDeleted = markDocumentDeleted;
exports.createDetailLink = createDetailLink;
exports.folderNameFromOpts = folderNameFromOpts;

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var _immutable = require('immutable');

/*
 * Returns immutablejs `List` from payload
 */

function toList(payload) {
  return (0, _immutable.List)(payload.rows.map(function (row) {
    return (0, _immutable.fromJS)(row.doc);
  }));
}

/**
 * findDocById
 *
 * @param {immutable.List} list
 * @param {string} docId
 * @returns {number}
 */

function findDocById(list, docId) {
  return list.findIndex(function (item) {
    return item.get('_id') === docId;
  });
}

function findDoc(list, doc) {
  return findDocById(list, doc.get('_id'));
}

function deleteDocFromList(list, doc) {
  var index = findDoc(list, doc);
  if (index === -1) {
    return list;
  }
  return list['delete'](index);
}

function getFoldersFromState(state) {
  return state.get('folders');
}

function getFolderFromState(state, folder) {
  return getFoldersFromState(state).get(folder);
}

function getDocumentsFromState(state) {
  return state.get('documents');
}

function saveFolderVars(state, folder, vars) {
  return state.setIn(['folders', folder, 'vars'], (0, _immutable.fromJS)(vars));
}

function getFolderVars(state, folder) {
  return state.getIn(['folders', folder, 'vars']);
}

function saveIdsInFolder(state, folder, ids) {
  return state.setIn(['folders', folder, 'ids'], ids);
}

function getIdsFromFolder(state, folder) {
  return state.getIn(['folders', folder, 'ids']);
}

function hasFolder(state, folder) {
  return state.hasIn(['folders', folder]);
}

function invalidateFolders(state) {
  return state.set('folders', (0, _immutable.Map)());
}

/**
 * getDocumentsInFolder returns documents for ids in given folder.
 */

function getDocumentsInFolder(state, folder) {
  var ids = getIdsFromFolder(state, folder);
  var documents = state.get('documents');
  return ids.map(function (docId) {
    return documents.get(docId);
  });
}

/**
 * removeDocument from all folders and documents.
 */

function removeDocument(state, id) {
  var newState = state.deleteIn(['documents', id]);
  newState.get('folders').map(function (folder, k) {
    var list = folder.get('ids');
    var index = list.indexOf(id);
    if (index != -1) {
      newState = newState.setIn(['folders', k, 'ids'], list['delete'](index));
    }
  });
  return newState;
}

function getDocument(state, id) {
  return state.get('documents').get(id);
}

function setDocument(state, doc) {
  return state.setIn(['documents', doc._id], (0, _immutable.fromJS)(doc));
}

/**
 * setDocuments merges documents in rows with existing documents.
 */

function setDocuments(state, rows) {
  var documents = (0, _immutable.Map)(rows.map(function (row) {
    return [row.id, (0, _immutable.fromJS)(row.doc)];
  }));
  return state.mergeIn(['documents'], documents);
}

function setQueryPayloadInState(state, folder, payload, folderVars) {
  var rows = payload.rows;

  var folderVarsFromQuery = _objectWithoutProperties(payload, ['rows']);

  var ids = (0, _immutable.List)(rows.map(function (row) {
    return row.id;
  }));
  return setDocuments(saveFolderVars(saveIdsInFolder(state, folder, ids), folder, _extends({}, folderVarsFromQuery, folderVars)), payload.rows);
}

/*
 * Replaces document `doc` in a given `list`
 *
 * Return original `list` if `doc` is not found
 */

function setDocInList(list, doc) {
  var index = findDoc(list, doc);
  if (index === -1) {
    return list;
  }
  return list.set(index, doc);
}

/**
 * Returns object from state
 *
 * @param {object} state
 * @param {string} mountPoint
 * @param {object} docId
 * @returns {immutable.Map}
 */
var getObjectFromState = function getObjectFromState(state, mountPoint, docId) {
  return state[mountPoint].get('documents').get(docId);
};

exports.getObjectFromState = getObjectFromState;
/**
 * createActionType
 *
 * @param {string} prefix
 * @param {string} action
 * @param {string} type
 * @returns {string}
 */

function createActionType(prefix, action, type) {
  return 'POUCHDB_' + prefix + '_' + action + '_' + type;
}

/**
 * markDocumentDeleted
 *
 * @param {immutable.Map} state
 * @param {string} docId
 * @returns {immutable.Map} new state
 */

function markDocumentDeleted(state, docId) {
  if (state.hasIn(['documents', docId])) {
    return state;
  }
  return state.setIn(['documents', docId, '_deleted'], true);
}

/**
 * createDetailLink
 *
 * @param urlPrefix
 * @param docId
 * @returns {string}
 */

function createDetailLink(urlPrefix, docId) {
  return urlPrefix + '/' + docId + '/';
}

/**
 * folderNameFromOpts
 *
 * @param options
 * @returns {string}
 */

function folderNameFromOpts() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  return JSON.stringify(options);
}